//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2003                 www.xilinx.com
//
//  XAPP 229 - Wide Block RAM in VirtexII and Spartan3 devices
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       top_v2_dp.vhd
//
//  Description :     Example top-level for a wide dual port memory in VirtexII
//
//  Date - revision : June 2nd 2006 - v 1.1
//
//  Author :          NJS
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors makeand you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specificallydisclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does notwarrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designswill be
//              uninterrupted or error free, or that defects in theDesigns
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regarding use or the results ofthe
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or forany
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on anytheory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure ofthe
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module top_v2_dp (
input 		rstin,			// reset input (active low)
input 		clkain,			// clock for port a
input 		clkbin,			// clock for port b (faster in this example)
output 		match,			// A to B transfer working
output 		locked) ;		// DCMs locked

wire 		high ;
wire 		low ;
wire 		clkaint ;
wire 		clkbint ;
wire 		clkax2 ;
wire 		clkbx2 ;
wire 		clkadcm ;
wire 		clkbdcm ;
wire 		clkax2dcm ;
wire 		clkbx2dcm ;
wire 		clka ;
wire 		clkb ;
wire 		locked_a ;
wire 		locked_b ;
wire 	[63:0] 	dia ;
wire 	[63:0] 	dib ;
wire 	[7:0] 	dipa ;
wire 	[7:0] 	dipb ;
reg 	[7:0] 	addra ;
reg 	[7:0] 	addrb ;
reg 		we_a ;
wire 		we_b ;
reg 		en_a ;
reg 		en_b ;
reg 		ssra ;
reg 		ssrb ;
wire 	[63:0] 	doa ;
wire 	[7:0] 	dopa ;
wire 	[63:0] 	dob ;
wire 	[7:0] 	dopb ;
wire 		rst ;
wire 		rstdcm ;
reg 	[71:0] 	data_to_write ;
reg 	[71:0] 	datab ;
reg 	[71:0] 	oldb ;
reg 	[3:0] 	counta ;
wire 		emptyab ;
reg 	[2:0]	matchab ;
reg 		emptyabret ;

assign low = 1'b0 ;
assign high = 1'b1 ;
assign locked = locked_a & locked_b ;
assign rst = ~rstin | ~(locked_a & locked_b) ;
assign rstdcm = ~rstin ;
assign match = matchab[2] ;

IBUFG ibufga (.I(clkain), .O(clkaint)) ;

DCM dcma (
        .CLKIN	(clkaint),
	.CLKFB	(clka),
	.RST	(rstdcm),
	.CLK0	(clkadcm),
	.CLK2X	(clkax2dcm),
	.LOCKED	(locked_a));

BUFG bufgac (.I(clkadcm), .O(clka)) ;
BUFG bufgacx2 (.I(clkax2dcm), .O(clkax2)) ;

IBUFG ibufgb (.I(clkbin), .O(clkbint)) ;

DCM dcmb (
        .CLKIN	(clkbint),
	.CLKFB	(clkb),
	.RST	(rstdcm),
	.CLK0	(clkbdcm),
	.CLK2X	(clkbx2dcm),
	.LOCKED	(locked_b));

BUFG bufgbd (.I(clkbdcm), .O(clkb)) ;
BUFG bufgbdx2 (.I(clkbx2dcm), .O(clkbx2)) ;

ramb16_s72_s72 ram0 (
	.dia	(dia),
	.dipa	(dipa),
	.dib	(dib),
	.dipb	(dipb),
	.en_a	(en_a),
	.en_b	(en_b),
	.wea	(we_a),
	.web	(we_b),
	.ssra	(ssra),
	.ssrb	(ssrb),
	.clka	(clka),
	.clkb	(clkb),
	.clkax2	(clkax2),
	.clkbx2	(clkbx2),
	.rst	(rst),
	.addra	(addra),
	.addrb	(addrb),
	.doa	(doa),
	.dopa	(dopa),
	.dob	(dob),
	.dopb	(dopb)) ;

assign dia = data_to_write[63:0] ;
assign dipa = data_to_write[71:64] ;

always @(posedge clka or posedge rst)				// set up a test to write patterns to port A
begin
if (rst) begin
	data_to_write <= 72'h000000000000000000 ;
	counta <= 4'b0000 ;
	addra <= 8'h00 ;
	we_a <= 1'b0 ;
	en_a <= 1'b0 ;
	ssra <= 1'b1 ;
end
else begin
	counta <= counta + 4'b0001 ;
	en_a <= 1'b1 ;
	ssra <= 1'b0 ;
	if (counta < 4'b1110) begin
		data_to_write <= {data_to_write[71:36] - 36'h000000001, data_to_write[35:0] + 36'h000000001} ;
		addra <= addra + 8'h01 ;
		we_a <= 1'b1 ;
	end else begin
		we_a <= 1'b0 ;
	end
end
end

assign we_b = 1'b0 ;
assign dib = 64'h0000000000000000 ;
assign dipb = 8'h00 ;

assign emptyab =(((addrb[7:6] == 2'b00) && (addra[7:6] == 2'b01)) |
		 ((addrb[7:6] == 2'b01) && (addra[7:6] == 2'b10)) |
		 ((addrb[7:6] == 2'b10) && (addra[7:6] == 2'b11)) |
		 ((addrb[7:6] == 2'b11) && (addra[7:6] == 2'b00)))
		 ? 1'b1 : 1'b0;

always @(posedge clkb or posedge rst)				// read and check port B
begin
if (rst) begin
	datab <= 72'h000000000000000000 ;
	oldb <= 72'h000000000000000000 ;
	addrb <= 8'b00000001 ;
	matchab <= 1'b0 ;
	en_b <= 1'b0 ;
	ssrb <= 1'b1 ;
	emptyabret <= 1'b0 ;
end
else begin
	emptyabret <= emptyab ;
	ssrb <= 1'b0 ;
	if (en_b) begin
		datab <= {dopb, dob} ;
		oldb <= datab ;
	end
	if (emptyab) begin
		en_b <= 1'b1 ;
		addrb <= addrb + 8'b00000001 ;
	end 
	else begin
		en_b <= 1'b0 ;
	end
	if (datab[35:0] == (oldb[35:0] + 36'b000000001)) begin
		matchab[0] <= 1'b1 ;
	end
	else begin
		matchab[0] <= 1'b0 ;
	end
	if (datab[71:36] == (oldb[71:36] - 36'b000000001)) begin
		matchab[1] <= 1'b1 ;
	end
	else begin
		matchab[1] <= 1'b0 ;
	end
	matchab[2] <= matchab[0] & matchab[1] ;
end
end

endmodule


