------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2003                 www.xilinx.com
--
--  XAPP 229 - Wide Block RAM in VirtexII and Spartan3 devices
--
------------------------------------------------------------------------------
--
--  File name :       ramb16_s72_s72.vhd
--
--  Description :     Wide (256 x 72) dual-port memory in VirtexII
--
--  Date - revision : June 2nd 2006 - v 1.1
--
--  Author :          NJS
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors makeand you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specificallydisclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does notwarrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designswill be
--              uninterrupted or error free, or that defects in theDesigns
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results ofthe
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or forany
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on anytheory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure ofthe
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

library unisim ;
use unisim.vcomponents.all ;

entity ramb16_s72_s72 is port (
	DIA    	 	: in std_logic_vector (63 downto 0);	-- A data in
	DIB    	 	: in std_logic_vector (63 downto 0);	-- B data in
	DIPA    	: in std_logic_vector (7 downto 0);	-- A parity in
	DIPB    	: in std_logic_vector (7 downto 0);	-- B parity in
	EN_A   		: in std_logic;				-- A EN in
	EN_B   		: in std_logic;				-- B EN in
	WEA    		: in std_logic;				-- A write enable in
	WEB    		: in std_logic;				-- B write enable in
	SSRA   		: in std_logic;				-- A sync set/reset in
	SSRB   		: in std_logic;				-- B sync set/reset in
	clka 		: in std_logic;				-- port A clock
	clkb 		: in std_logic;				-- port B clock
	clkax2 		: in std_logic;				-- port A clock x2
	clkbx2 		: in std_logic;				-- port B clock x2
	rst		: in std_logic;				-- reset
	ADDRA  		: in std_logic_vector (7 downto 0);	-- A address in
	ADDRB  		: in std_logic_vector (7 downto 0);	-- B address in
	DOA    		: out std_logic_vector (63 downto 0);	-- A data out
	DOB    		: out std_logic_vector (63 downto 0);	-- B data out
	DOPA    	: out std_logic_vector (7 downto 0);	-- A parity out
	DOPB    	: out std_logic_vector (7 downto 0));	-- B parity out
end ramb16_s72_s72;

architecture arch_ramb16_s72_s72 of ramb16_s72_s72 is

signal high 		: std_logic ;
signal low 		: std_logic ;
signal data_muxa 	: std_logic_vector (31 downto 0);
signal data_muxb 	: std_logic_vector (31 downto 0);
signal doa_int 		: std_logic_vector (31 downto 0);
signal dob_int 		: std_logic_vector (31 downto 0);
signal dopa_int 	: std_logic_vector (3 downto 0);
signal dopb_int 	: std_logic_vector (3 downto 0);
signal par_muxa 	: std_logic_vector (3 downto 0);
signal par_muxb		: std_logic_vector (3 downto 0);
signal addr_muxa 	: std_logic_vector (8 downto 0);
signal addr_muxb 	: std_logic_vector (8 downto 0);
signal muxa 		: std_logic;
signal muxb 		: std_logic;
signal en_a_del		: std_logic;
signal en_b_del		: std_logic;
signal clka_lac		: std_logic;
signal clkb_lac		: std_logic;
signal notclka		: std_logic;
signal notclkb		: std_logic;

begin

low <= '0' ;
high <= '1' ;
notclka <= not clka after 100 pS ;	-- solves a problem with simulation from MTI 6.0 on
notclkb <= not clkb after 100 pS ;	-- solves a problem with simulation from MTI 6.0 on

-- Generate logic available clocks
clkac_lacff : fdc port map (c => clkax2, d => notclka, clr => rst, q => clka_lac);
clkbd_lacff : fdc port map (c => clkbx2, d => notclkb, clr => rst, q => clkb_lac);

block_ram : ramb16_s36_s36 port map (
	DIA    		=> data_muxa ,
	DIB    		=> data_muxb,
	DIPA    	=> par_muxa,
	DIPB    	=> par_muxb,
	ENA    		=> en_a,
	ENB    		=> en_b,
	WEA    		=> wea,
	WEB    		=> web,
	SSRA   		=> ssra,
	SSRB   		=> ssrb,
	clkA   		=> clkax2,
	clkB   		=> clkbx2,
	ADDRA  		=> addr_muxa,
	ADDRB  		=> addr_muxb,
	DOA    		=> doa_int,
	DOB    		=> dob_int,
	DOPA    	=> dopa_int,
	DOPB    	=> dopb_int);

data_muxa <= dia(63 downto 32) when muxa = '0' else dia(31 downto 0) ;
data_muxb <= dib(63 downto 32) when muxb = '0' else dib(31 downto 0) ;

par_muxa <= dipa(7 downto 4) when muxa = '0' else dipa(3 downto 0) ;
par_muxb <= dipb(7 downto 4) when muxb = '0' else dipb(3 downto 0) ;

addr_muxa <= ('1' & addra) when muxa = '0' else ('0' & addra) ;
addr_muxb <= ('1' & addrb) when muxb = '0' else ('0' & addrb) ;

muxa <= clka_lac ;
		
process (clka, rst)
begin
if rst = '1' then
	doa(31 downto 0) <= (others => '0') ;
	dopa(3 downto 0) <= (others => '0') ;
elsif clka'event and clka = '1' then
	if en_a = '1' then
		doa(31 downto 0) <= doa_int ;
		dopa(3 downto 0) <= dopa_int ;
	end if ;
end if ;
end process ;

process (clka, rst)
begin
if rst = '1' then
	doa(63 downto 32) <= (others => '0') ;
	dopa(7 downto 4) <= (others => '0') ;
	en_a_del <= '0' ;
elsif clka'event and clka = '0' then
	en_a_del <= en_a ;
	if en_a_del = '1' then
		doa(63 downto 32) <= doa_int ;
		dopa(7 downto 4) <= dopa_int ;
	end if ;
end if ;
end process ;

muxb <= clkb_lac ;

process (clkb, rst)
begin
if rst = '1' then
	dob(31 downto 0) <= (others => '0') ;
	dopb(3 downto 0) <= (others => '0') ;
elsif clkb'event and clkb = '1' then
	if en_b = '1' then
		dob(31 downto 0) <= dob_int ;
		dopb(3 downto 0) <= dopb_int ;
	end if ;
end if ;
end process ;

process (clkb, rst)
begin
if rst = '1' then
	dob(63 downto 32) <= (others => '0') ;
	dopb(7 downto 4) <= (others => '0') ;
	en_b_del <= '0' ;
elsif clkb'event and clkb = '0' then
	en_b_del <= en_b ;
	if en_b_del = '1' then
		dob(63 downto 32) <= dob_int ;
		dopb(7 downto 4) <= dopb_int ;
	end if ;
end if ;
end process ;

end arch_ramb16_s72_s72;


