------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2003                 www.xilinx.com
--
--  XAPP 229 - Wide Block RAM in VirtexII and Spartan3 devices
--
------------------------------------------------------------------------------
--
--  File name :       top_v2_dp.vhd
--
--  Description :     Example top-level for a wide dual port memory in VirtexII
--
--  Date - revision : 2nd June 2006 - v 1.1
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors makeand you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specificallydisclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does notwarrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designswill be
--              uninterrupted or error free, or that defects in theDesigns
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results ofthe
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or forany
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on anytheory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure ofthe
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

library unisim ;
use unisim.vcomponents.all ;

entity top_v2_dp is port (
	rstin		: in std_logic ;	-- reset input (active low)
	clkain 		: in std_logic;		-- clock for port A
	clkbin 		: in std_logic;		-- clock for port B
	match		: out std_logic ;	-- A to B transfer working
	locked		: out std_logic);	-- DCMs locked
end top_v2_dp;

architecture arch_top_v2_dp of top_v2_dp is
-- components
-- wide dual port RAM - 256 x 72 max
component ramb16_s72_s72 port (
	DIA    	 	: in std_logic_vector (63 downto 0);	-- A data in
	DIB    	 	: in std_logic_vector (63 downto 0);	-- B data in
	DIPA    	: in std_logic_vector (7 downto 0);	-- A parity in
	DIPB    	: in std_logic_vector (7 downto 0);	-- B parity in
	EN_A   		: in std_logic;				-- A EN in
	EN_B   		: in std_logic;				-- B EN in
	WEA    		: in std_logic;				-- A write enable in
	WEB    		: in std_logic;				-- B write enable in
	SSRA   		: in std_logic;				-- A sync set/reset in
	SSRB   		: in std_logic;				-- B sync set/reset in
	clka 		: in std_logic;				-- port A clock
	clkb 		: in std_logic;				-- port B clock
	clkax2 		: in std_logic;				-- port A clock x2
	clkbx2 		: in std_logic;				-- port B clock x2
	rst		: in std_logic;				-- reset
	ADDRA  		: in std_logic_vector (7 downto 0);	-- A address in
	ADDRB  		: in std_logic_vector (7 downto 0);	-- B address in
	DOA    		: out std_logic_vector (63 downto 0);	-- A data out
	DOB    		: out std_logic_vector (63 downto 0);	-- B data out
	DOPA    	: out std_logic_vector (7 downto 0);	-- A parity out
	DOPB    	: out std_logic_vector (7 downto 0));	-- B parity out
end component ;

signal high 		: std_logic ;
signal low 		: std_logic ;
signal clkaint 		: std_logic ;
signal clkbint 		: std_logic ;
signal clkax2 		: std_logic ;
signal clkbx2 		: std_logic ;
signal clkadcm 		: std_logic ;
signal clkbdcm 		: std_logic ;
signal clkax2dcm 	: std_logic ;
signal clkbx2dcm 	: std_logic ;
signal clka 		: std_logic ;
signal clkb		: std_logic ;
signal locked_a		: std_logic ;
signal locked_b		: std_logic ;
signal dia 		: std_logic_vector (63 downto 0) ;
signal dib 		: std_logic_vector (63 downto 0) ;
signal dipa 		: std_logic_vector (7 downto 0) ;
signal dipb 		: std_logic_vector (7 downto 0) ;
signal addra 		: std_logic_vector (7 downto 0) ;
signal addrb 		: std_logic_vector (7 downto 0) ;
signal we_a 		: std_logic ;
signal we_b 		: std_logic ;
signal en_a 		: std_logic ;
signal en_b 		: std_logic ;
signal ssra 		: std_logic ;
signal ssrb 		: std_logic ;
signal dob 		: std_logic_vector (63 downto 0) ;
signal dopb 		: std_logic_vector (7 downto 0) ;
signal rst 		: std_logic ;
signal rstdcm 		: std_logic ;
signal data_to_write 	: std_logic_vector (71 downto 0) ;
signal datab		: std_logic_vector (71 downto 0) ;
signal oldb		: std_logic_vector (71 downto 0) ;
signal counta		: std_logic_vector (3 downto 0) ;
signal emptyab 		: std_logic ;
signal emptyabret	: std_logic ;
signal matchab		: std_logic_vector (5 downto 0) ;

begin

low <= '0' ;
high <= '1' ;
rst <= not rstin or not (locked_a and locked_b) ;
rstdcm <= not rstin ;
locked <= locked_a and locked_b ;
match <= matchab(5) ;

ibufga : ibufg port map (i => clkain, o => clkaint) ;

dcma: dcm port map (
        clkin   	=> clkaint,
	CLKFB   	=> clka,
	DSSEN 		=> low,
	PSINCDEC	=> low,
	PSEN 		=> low,
	PSCLK 		=> low,
	RST     	=> rstdcm,
	CLK0    	=> clkadcm,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> clkax2dcm,
	CLK2X180	=> open,
	CLKDV   	=> open,
	CLKFX   	=> open,
	CLKFX180	=> open,
	LOCKED  	=> locked_a,
	PSDONE  	=> open,
	STATUS  	=> open);

bufga : bufg port map (i => clkadcm, o	=> clka) ;
bufgax2 : bufg port map (i => clkax2dcm, o => clkax2) ;

ibufgb : ibufg port map (i => clkbin, o => clkbint) ;

dcmb: dcm port map (
        clkin   	=> clkbint,
	CLKFB   	=> clkb,
	DSSEN 		=> low,
	PSINCDEC	=> low,
	PSEN 		=> low,
	PSCLK 		=> low,
	RST     	=> rstdcm,
	CLK0    	=> clkbdcm,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> clkbx2dcm,
	CLK2X180	=> open,
	CLKDV   	=> open,
	CLKFX   	=> open,
	CLKFX180	=> open,
	LOCKED  	=> locked_b,
	PSDONE  	=> open,
	STATUS  	=> open);

bufgb : bufg port map (i => clkbdcm, o	=> clkb) ;
bufgbx2 : bufg port map (i => clkbx2dcm, o => clkbx2) ;

ram0 : ramb16_s72_s72 port map (
	DIA    		=> dia,
	DIB    		=> dib,
	DIPA    	=> dipa,
	DIPB    	=> dipb,
	EN_A   		=> en_a,
	EN_B   		=> en_b,
	WEA    		=> we_a,
	WEB    		=> we_b,
	SSRA   		=> ssra,
	SSRB   		=> ssrb,
	clka 		=> clka,
	clkb		=> clkb,
	clkax2 		=> clkax2,
	clkbx2		=> clkbx2,
	rst		=> rst,
	ADDRA  		=> addra,
	ADDRB  		=> addrb,
	DOA    		=> open,
	DOB    		=> dob,
	DOPA    	=> open,
	DOPB    	=> dopb) ;

dia <= data_to_write(63 downto 0) ;
dipa <= data_to_write(71 downto 64) ;

process (clka, rst)				-- set up a test to write patterns to port A
begin
if rst = '1' then
	data_to_write <= (others => '0') ;
	counta <= "0000" ;
	addra <= "00000000" ;
	we_a <= '0' ;
	en_a <= '0' ;
	ssra <= '1' ;
elsif clka'event and clka = '1' then
	counta <= counta + 1 ;
	en_a <= '1' ;
	ssra <= '0' ;
	if counta < "1101" then
		data_to_write <= (data_to_write(71 downto 64) + 3) & (data_to_write(63 downto 48) - 2) &
				 (data_to_write(47 downto 32) + 2) & (data_to_write(31 downto 16) -1) &
				 (data_to_write(15 downto 0) + 1) ;
		addra <= addra + 1 ;
		we_a <= '1' ;
	else
		we_a <= '0' ;
	end if ;
end if ;
end process ;

we_b <= '0' ;
dib <= (others => '0') ;
dipb <= "00000000" ;

emptyab <= '1' when addrb(7 downto 6) = "00" and addra(7 downto 6) = "01" else
           '1' when addrb(7 downto 6) = "01" and addra(7 downto 6) = "10" else
           '1' when addrb(7 downto 6) = "10" and addra(7 downto 6) = "11" else
           '1' when addrb(7 downto 6) = "11" and addra(7 downto 6) = "00" else
           '0' ;

process (clkb, rst)				-- read and check port B
begin
if rst = '1' then
	datab <= (others => '0') ;
	oldb <= (others => '0') ;
	addrb <= "00000000" ;
	matchab <= "000000" ;
	en_b <= '0' ;
	ssrb <= '1' ;
	emptyabret <= '0' ;
elsif clkb'event and clkb = '1' then
	emptyabret <= emptyab ;
	ssrb <= '0' ;
	if en_b = '1' then
		datab <= dopb & dob ;
		oldb <= datab ;
	end if ;
	if emptyab = '1' then
		en_b <= '1' ;
		addrb <= addrb + 1 ;
	else
		en_b <= '0' ;
	end if ;
	if datab(15 downto 0) = oldb(15 downto 0) + 1 then
		matchab(0) <= '1' ;
	else
		matchab(0) <= '0' ;
	end if ;
	if datab(31 downto 16) = oldb(31 downto 16) - 1 then
		matchab(1) <= '1' ;
	else
		matchab(1) <= '0' ;
	end if ;
	if datab(47 downto 32) = oldb(47 downto 32) + 2 then
		matchab(2) <= '1' ;
	else
		matchab(2) <= '0' ;
	end if ;
	if datab(63 downto 48) = oldb(63 downto 48) - 2 then
		matchab(3) <= '1' ;
	else
		matchab(3) <= '0' ;
	end if ;
	if datab(71 downto 64) = oldb(71 downto 64) + 3 then
		matchab(4) <= '1' ;
	else
		matchab(4) <= '0' ;
	end if ;
	matchab(5) <= matchab(0) and matchab(1) and matchab(2) and matchab(3) and matchab(4) ;
end if ;
end process ;

end arch_top_v2_dp;


