//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2003                 www.xilinx.com
//
//  XAPP 229 - Wide Block RAM in VirtexII and Spartan3 devices
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       top_v2_sp.vhd
//
//  Description :     Example top-level for a wide single port memory in VirtexII
//
//  Date - revision : 2nd June 2006 - v 1.1
//
//  Author :          NJS
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors makeand you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specificallydisclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does notwarrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designswill be
//              uninterrupted or error free, or that defects in theDesigns
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regarding use or the results ofthe
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or forany
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on anytheory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure ofthe
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module top_v2_sp (
input 		rstin,			// reset input (active low)
input 		clkin,			// clock 
output 		match,			// transfer working
output 		locked) ;		// DCM locked

wire 		high ;
wire 		low ;
wire 		clkint ;
wire 		clkx2 ;
wire 		clkdcm ;
wire 		clkx2dcm ;
wire 		clk ;
wire 	[127:0] di ;
wire 	[15:0] 	dip ;
reg 	[6:0] 	addr ;
reg 	[6:0] 	read_addr ;
reg 	[6:0] 	write_addr ;
reg 		we ;
reg 		en ;
reg 		ssr ;
wire 	[127:0] do ;
wire 	[15:0] 	dop ;
wire 		rst ;
wire 		rstdcm ;
reg 	[143:0] data_to_write ;
reg 	[143:0] oldb ;
reg 	[3:0] 	counta ;
reg 	[8:0]	matchab ;
wire		lockedint ;
reg 	[143:0]	ramdata ;

assign low = 1'b0 ;
assign high = 1'b1 ;
assign locked = lockedint ;
assign rst = ~rstin | ~lockedint ;
assign rstdcm = ~rstin ;
assign match = matchab[8] ;

IBUFG ibufga (.I(clkin), .O(clkint)) ;

DCM dcma (
        .CLKIN	(clkint),
	.CLKFB	(clk),
	.RST	(rstdcm),
	.CLK0	(clkdcm),
	.CLK2X	(clkx2dcm),
	.LOCKED	(lockedint));

BUFG clk_bufg (.I(clkdcm), .O(clk)) ;
BUFG CLK_bufgx2 (.I(clkx2dcm), .O(clkx2)) ;

ramb16_s144 ram0 (
	.di	(di),
	.dip	(dip),
	.en	(en),
	.we	(we),
	.ssr	(ssr),
	.clk	(clk),
	.clkx2	(clkx2),
	.rst	(rst),
	.addr	(addr),
	.do	(do),
	.dop	(dop)) ;

assign di = data_to_write[127:0] ;
assign dip = data_to_write[143:128] ;

always @(posedge clk or posedge rst)				// set up a test to write patterns to port A
begin
if (rst) begin
	data_to_write <= 144'h000000000000000000000000000000000000 ;
	counta <= 4'b0000 ;
	addr <= 7'b0000000 ;
	write_addr <= 7'b0000000 ;
	read_addr <= 7'b0000000 ;
	we <= 1'b0 ;
	en <= 1'b0 ;
	ssr <= 1'b1 ;
	matchab <= 9'b000000000 ;
end
else begin
	ramdata <= {dop, do} ;
	counta <= counta + 4'b0001 ;
	en <= 1'b1 ;
	ssr <= 1'b0 ;
	if (counta < 4'h8) begin
		data_to_write <= {data_to_write[143:126] - 18'h4, data_to_write[125:108] + 18'h4,
				  data_to_write[107:90] - 18'h3,  data_to_write[89:72] + 18'h3,
				  data_to_write[71:54] - 18'h2,   data_to_write[53:36] + 18'h2,
				  data_to_write[35:18] - 18'h1,   data_to_write[17:0] + 18'h1} ;
		write_addr <= write_addr + 7'b0000001 ;
		addr <= write_addr ;
		we <= 1'b1 ;
	end 
	else begin
		we <= 1'b0 ;
		read_addr <= read_addr + 7'b0000001 ;
		addr <= read_addr ;
	end
	if (counta > 4'hA || counta == 4'h0 || counta == 4'h1 || counta == 4'h2) begin
		oldb <= ramdata ;
		if (ramdata[143:126] == oldb[143:126] - 18'h4) begin
			matchab[0] <= 1'b1 ;
		end
		else begin
			matchab[0] <= 1'b0 ;
		end
		if (ramdata[125:108] == oldb[125:108] + 18'h4) begin
			matchab[1] <= 1'b1 ;
		end
		else begin
			matchab[1] <= 1'b0 ;
		end
		if (ramdata[107:90] == oldb[107:90] - 18'h3) begin
			matchab[2] <= 1'b1 ;
		end
		else begin
			matchab[2] <= 1'b0 ;
		end
		if (ramdata[89:72] == oldb[89:72] + 18'h3) begin
			matchab[3] <= 1'b1 ;
		end
		else begin
			matchab[3] <= 1'b0 ;
		end
		if (ramdata[71:54] == oldb[71:54] - 18'h2) begin
			matchab[4] <= 1'b1 ;
		end
		else begin
			matchab[4] <= 1'b0 ;
		end
		if (ramdata[53:36] == oldb[53:36] + 18'h2) begin
			matchab[5] <= 1'b1 ;
		end
		else begin
			matchab[5] <= 1'b0 ;
		end
		if (ramdata[35:18] == oldb[35:18] - 18'h1) begin
			matchab[6] <= 1'b1 ;
		end
		else begin
			matchab[6] <= 1'b0 ;
		end
		if (ramdata[17:0] == oldb[17:0] + 18'h1) begin
			matchab[7] <= 1'b1 ;
		end
		else begin
			matchab[7] <= 1'b0 ;
		end
	end
	matchab[8] <= matchab[7] & matchab[6] & matchab[5] & matchab[4] & matchab[3] & matchab[2] & matchab[1] & matchab[0] ;
end
end

endmodule


