------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2003                 www.xilinx.com
--
--  XAPP 229 - Wide Block RAM in VirtexII and Spartan3 devices
--
------------------------------------------------------------------------------
--
--  File name :       top_v2_sp.vhd
--
--  Description :     Example top-level for wide single port memory in VirtexII
--
--  Date - revision : 2nd June 2006 - v 1.1
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors makeand you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specificallydisclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does notwarrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designswill be
--              uninterrupted or error free, or that defects in theDesigns
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results ofthe
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or forany
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on anytheory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure ofthe
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

library unisim ;
use unisim.vcomponents.all ;

entity top_v2_sp is port (
	rstin		: in std_logic ;	-- reset input (active low)
	clkin 		: in std_logic;		-- clock 
	match		: out std_logic ;	-- transfer working
	locked		: out std_logic);	-- DCMs locked
end top_v2_sp;

architecture arch_top_v2_sp of top_v2_sp is
-- components
-- wide single port RAM - 128 x 144 max
component ramb16_s144 port (
	DI    	 	: in std_logic_vector (127 downto 0);	-- data in
	DIP    		: in std_logic_vector (15 downto 0);	-- parity in
	en   		: in std_logic;				-- EN in
	WE    		: in std_logic;				-- write enable in
	ssr   		: in std_logic;				-- sync set/reset in
	clk 		: in std_logic;				-- clock
	clkx2 		: in std_logic;				-- clock x2
	rst		: in std_logic;				-- reset
	ADDR  		: in std_logic_vector (6 downto 0);	-- address in
	DO    		: out std_logic_vector (127 downto 0);	-- data out
	DOP    		: out std_logic_vector (15 downto 0));	-- parity out
end component ;

signal high 		: std_logic ;
signal low 		: std_logic ;
signal clkint 		: std_logic ;
signal clkx2 		: std_logic ;
signal clkdcm 		: std_logic ;
signal clkx2dcm 	: std_logic ;
signal clk 		: std_logic ;
signal di 		: std_logic_vector (127 downto 0) ;
signal dip 		: std_logic_vector (15 downto 0) ;
signal addr 		: std_logic_vector (6 downto 0) ;
signal read_addr 	: std_logic_vector (6 downto 0) ;
signal write_addr 	: std_logic_vector (6 downto 0) ;
signal we 		: std_logic ;
signal en 		: std_logic ;
signal ssr 		: std_logic ;
signal do 		: std_logic_vector (127 downto 0) ;
signal dop 		: std_logic_vector (15 downto 0) ;
signal rst 		: std_logic ;
signal rstdcm 		: std_logic ;
signal data_to_write 	: std_logic_vector (143 downto 0) ;
signal ramdata		: std_logic_vector (143 downto 0) ;
signal oldb		: std_logic_vector (143 downto 0) ;
signal counta		: std_logic_vector (3 downto 0) ;
signal matchab		: std_logic_vector (8 downto 0) ;
signal lockedint	: std_logic ;

begin

low <= '0' ;
high <= '1' ;
rst <= not rstin or not lockedint ;
rstdcm <= not rstin ;
locked <= lockedint ;
match <= matchab(8) ;

ibufga : ibufg port map (i => clkin, o => clkint) ;

dcma: dcm port map (
        clkin   	=> clkint,
	CLKFB   	=> clk,
	DSSEN 		=> low,
	PSINCDEC	=> low,
	PSEN 		=> low,
	PSCLK 		=> low,
	RST     	=> rstdcm,
	CLK0    	=> clkdcm,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> clkx2dcm,
	CLK2X180	=> open,
	CLKDV   	=> open,
	CLKFX   	=> open,
	CLKFX180	=> open,
	LOCKED  	=> lockedint,
	PSDONE  	=> open,
	STATUS  	=> open);

clk_bufg : bufg port map (i => clkdcm, o => clk) ;
bufgx2 : bufg port map (i => clkx2dcm, o => clkx2) ;

ram0 : ramb16_s144 port map (
	DI    		=> di,
	DIP    		=> dip,
	EN   		=> en,
	WE   		=> we,
	SSR   		=> ssr,
	clk 		=> clk,
	clkx2 		=> clkx2,
	rst		=> rst,
	ADDR  		=> addr,
	DO    		=> do,
	DOP    		=> dop) ;

di <= data_to_write(127 downto 0) ;
dip <= data_to_write(143 downto 128) ;

process (clk, rst)				-- set up a test to write patterns
begin
if rst = '1' then
	ramdata <= (others => '0') ;
	data_to_write <= (others => '0') ;
	oldb <= (others => '0') ;
	counta <= "0000" ;
	addr <= "0000000" ;
	read_addr <= "0000000" ;
	write_addr <= "0000000" ;
	we <= '0' ;
	en <= '0' ;
	ssr <= '1' ;
	matchab <= "000000000" ;
elsif clk'event and clk = '1' then
	ramdata <= dop & do ;
	counta <= counta + 1 ;
	en <= '1' ;
	ssr <= '0' ;
	if counta < "1000" then
		data_to_write <= (data_to_write(143 downto 126) - 4) & (data_to_write(125 downto 108) + 4) & 
		                 (data_to_write(107 downto 90) - 3)  & (data_to_write(89 downto 72) + 3)   &
		                 (data_to_write(71 downto 54) - 2)   & (data_to_write(53 downto 36) + 2)   &
		                 (data_to_write(35 downto 18) - 1)   & (data_to_write(17 downto 0) + 1)   ;
		write_addr <= write_addr + 1 ;
		addr <= write_addr ;
		we <= '1' ;
	else
		we <= '0' ;
		read_addr <= read_addr + 1 ;
		addr <= read_addr ;
	end if ;
	if counta > "1010" or counta = "0000" or counta = "0001" or counta = "0010" then
		oldb <= ramdata ;
		if ramdata(143 downto 126) = oldb(143 downto 126) - 4 then
			matchab(0) <= '1' ;
		else
			matchab(0) <= '0' ;
		end if ;
		if ramdata(125 downto 108) = oldb(125 downto 108) + 4 then
			matchab(1) <= '1' ;
		else
			matchab(1) <= '0' ;
		end if ;
		if ramdata(107 downto 90) = oldb(107 downto 90) - 3 then
			matchab(2) <= '1' ;
		else
			matchab(2) <= '0' ;
		end if ;
		if ramdata(89 downto 72) = oldb(89 downto 72) + 3 then
			matchab(3) <= '1' ;
		else
			matchab(3) <= '0' ;
		end if ;
		if ramdata(71 downto 54) = oldb(71 downto 54) - 2 then
			matchab(4) <= '1' ;
		else
			matchab(4) <= '0' ;
		end if ;
		if ramdata(53 downto 36) = oldb(53 downto 36) + 2 then
			matchab(5) <= '1' ;
		else
			matchab(5) <= '0' ;
		end if ;
		if ramdata(35 downto 18) = oldb(35 downto 18) - 1 then
			matchab(6) <= '1' ;
		else
			matchab(6) <= '0' ;
		end if ;
		if ramdata(17 downto 0) = oldb(17 downto 0) + 1 then
			matchab(7) <= '1' ;
		else
			matchab(7) <= '0' ;
		end if ;
	end if ;
	matchab(8) <= matchab(0) and matchab(1) and matchab(2) and matchab(3) and matchab(4) and matchab(5) and matchab(6) and matchab(7) ;
end if ;
end process ;
end arch_top_v2_sp;


